--[[
Copyright 2012 João Cardoso
PetTracker is distributed under the terms of the GNU General Public License (Version 3).
As a special exception, the copyright holders of this addon do not give permission to
redistribute and/or modify it.

This addon is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the addon. If not, see <http://www.gnu.org/licenses/gpl-3.0.txt>.

This file is part of PetTracker.
--]]

local ADDON, Addon = ...
local Blip = Addon:NewClass('Button', 'Blip', 'WorldMapUnitTemplate')
local Drop = CreateFrame('Frame', ADDON..'BlipDrop', nil, 'UIDropDownMenuTemplate')

Drop.point, Drop.relativePoint = 'TOP', 'BOTTOM'
Blip.Drop = Drop


--[[ Events ]]--

function Blip:OnCreate()
	self:RegisterForClicks('LeftButtonUp', 'RightButtonUp')
	self:SetScript('OnClick', self.OnClick)
	self:SetSize(13, 13)
end

function Blip:OnRelease()
	self:Hide()
end

function Blip:OnClick(button)
	if button == 'LeftButton' then
		self:GoJournal()
	else
		self:ShowDrop()
	end
end


--[[ Actions ]]--

function Blip:GoJournal()
	PetJournal_LoadUI()
	HideUIPanel(WorldMapFrame)
	ShowUIPanel(PetJournalParent)

	self:Select()
end

function Blip:AddWaypoint()
	TomTom:AddMFWaypoint(GetCurrentMapAreaID(), 0, self.x, self.y)
end


--[[ Dropdown ]]--

function Blip:ShowDrop()
	self.lines = {}
	self:AddLine(self:GetTooltip(), false)
	self:AddLine('Show in Journal', 'GoJournal')

	if IsAddOnLoaded('TomTom') then
		self:AddLine('Add Waypoint', 'AddWaypoint')
	end

	EasyMenu(self.lines, Drop, self, 0, 0)
end

function Blip:AddLine(text, action)
	tinsert(self.lines, {
		text = text,
		notCheckable = 1,
		isTitle = not action,
		func = function()
			self[action](self)
		end
	})
end